package com.chinaitop.depot.storage.service;

import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSONArray;
import com.chinaitop.depot.storage.model.StorageQualitycheck;
import com.chinaitop.depot.storage.model.StorageQualitycheckExample;
import com.chinaitop.depot.storage.model.StorageQualitycheckInspectionExample;
import com.chinaitop.depot.storage.model.StorageQualitycheckSon;

public interface StorageQualitycheckService {

	/**
	 * 按条件查询qualitycheck列表
	 *
	 * @param example
	 * @return
	 */
	List<StorageQualitycheck> findByCondition(StorageQualitycheckExample example);

	/**
	 * 条件查询qualitycheck列表
	 *
	 * @param example
	 * @return
	 */
	List<StorageQualitycheck> queryByQualitycheck(StorageQualitycheck example);

	/**
	 * 新增qualitycheck信息
	 *
	 * @param qualitycheck
	 */
	void save(StorageQualitycheck qualitycheck, String qualityInspection);

	/**
	 * 新增qualitycheck字表信息
	 *
	 * @param qualitycheckone
	 */
	void saveOne(StorageQualitycheckSon qualitycheckone);

	/**
	 * 根据id查询数据
	 *
	 * @param id
	 * @return
	 */
	Map<String, Object> loadDataById(Integer id);

	/**
	 * 根据主键id删除数据
	 *
	 * @param id
	 */
	void remove(int id);

    /**
     * @param qualitycheck
     * @return
     */
    List<StorageQualitycheck> getHouseOrWareList(
            StorageQualitycheck qualitycheck);

    /**
     * @param id
     * @return
     */
    String getQualityStandardStr(Integer id);

	/**
	 * 查询第三方检查列表
	 * @param qualitycheck
	 * @return
	 */
	List<StorageQualitycheck> getThirdCheckList(StorageQualitycheck qualitycheck);
	/**
	 * 查询粮情专卡中粮食质量检测记录
	 * @param qualitycheck
	 * @return
	 */
	List<StorageQualitycheck> getQualitycheckListByFoodbasicinfoCheck(StorageQualitycheck qualitycheck,Integer foodbasicinfoId);

	void setQualityCheckByHistoryStatus (Integer houseId, Integer warehouseId);
	List<Map<String,Object>> getStoreQualityList(Integer orgId);
	List<Map<String,Object>> getTrdStoreQualityList();

	List<StorageQualitycheck> getDataByZJId(Map<String, Object> paramMap);


	/**
	 * 取出封仓需要的数据
	 * @param qualityCheckExample
	 * @return
	 */
	List<StorageQualitycheck> queryByQualitycheckData(StorageQualitycheck qualityCheckExample);

	/**
	 * 查看business_store_ware_detail中是否有数据
	 * @param qualityCheckExample
	 * @return
	 */
	List<StorageQualitycheck> queryByQualitycheckusiness(StorageQualitycheck qualityCheckExample);

	/**
	 * 库点同步 - 质检信息主表
	 * @return
	 */
    List<Map<String,Object>> getAllQualityMain(Map<String, Object> paramMap);

	/**
	 * 库点同步 - 质检信息明细
	 * @return
	 */
	List<Map<String, Object>> getAllQualityInfo(Map<String, Object> paramMap);

	/**
	 * 库点同步 - 粮食库存
	 * @return
	 */
    List<Map<String, Object>> getAllFoodStocks(Map<String, Object> paramMap);

    /**
     * 把质检数据推送给市平台
     * @param qualitycheck
     * @param reportPath 接口地址
     * @throws Exception
     */
    void pushQualitycheck_spt(StorageQualitycheck qualitycheck, String reportPath) throws Exception;

    /**
     * 把粮食初检的粮食等级推送给出入库
     * @param qualitycheck
     * @param ws_path webservice地址
     * @throws Exception
     */
    void pushQualitycheckLsdj_crk(StorageQualitycheck qualitycheck, String ws_path) throws Exception;


    //获取仓房
    JSONArray getHouseIdAll(Integer orgId);

	Map<String, Object> getcheckByIds(Integer id);

	int deleteStorageQualitycheckInspection(StorageQualitycheckInspectionExample example);

	/**
	 * 按条件查询质检数据列表
	 * @param houseId 仓房ID
	 * @param wareId 货位ID
	 * @param checkType 检验类型(0:自检,3:出库检验,4:春秋普查,5:第三方检查)
	 * @param historyStatus 数据状态(0:新数据,1:已归档数据)
	 * @param orgId 组织机构ID
	 * @param processState 1扦样化验待处理 2化验已处理
	 * @param isdx 0:大于,1小于
	 * @param isdx 0:大于,1小于
	 * @param checkTime 检验时间
	 * @return
	 * @throws Exception
	 */
	List<StorageQualitycheck> getList(Integer houseId, Integer wareId, String checkType, String checkResult,
			Integer historyStatus, Integer orgId, Integer processState, String isdx, String checkTime) throws Exception;

	/**
	 * 按条件查询质检数据详情
	 * @param houseId 仓房ID
	 * @param wareId 货位ID
	 * @param checkType 检验类型(0:自检,3:出库检验,4:春秋普查,5:第三方检查)
	 * @param historyStatus 数据状态(0:新数据,1:已归档数据)
	 * @param orgId 组织机构ID
	 * @param processState 1扦样化验待处理 2化验已处理
	 * @return
	 * @throws Exception 
	 */
	Map<String, Object> queryCheckDataDetail(Integer houseId, Integer wareId, String checkType, 
			Integer historyStatus, Integer orgId, Integer processState) throws Exception;

	/**
	 * 查询一个货位在封仓后的出库检验数据
	 * @param houseId
	 * @param wareId
	 * @return
	 * @throws Exception
	 */
	Map<String, Object> findConditionCkCheck(Integer houseId, Integer wareId, String checkType) throws Exception;
}
