package com.chinaitop.depot.storage.service.impl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.feign.AgileHouseFeignClient;
import com.chinaitop.depot.feign.StorageHouseFeignClient;
import com.chinaitop.depot.storage.mapper.StorageQualitycheckInspectionMapper;
import com.chinaitop.depot.storage.mapper.StorageQualitycheckMapper;
import com.chinaitop.depot.storage.mapper.StorageQualitycheckSonMapper;
import com.chinaitop.depot.storage.model.StorageQualitycheck;
import com.chinaitop.depot.storage.model.StorageQualitycheckExample;
import com.chinaitop.depot.storage.model.StorageQualitycheckExample.Criteria;
import com.chinaitop.depot.storage.model.StorageQualitycheckInspection;
import com.chinaitop.depot.storage.model.StorageQualitycheckInspectionExample;
import com.chinaitop.depot.storage.model.StorageQualitycheckSon;
import com.chinaitop.depot.storage.service.StorageQualitycheckService;
import com.chinaitop.depot.storage.service.StorageQualitycheckSonService;
import com.chinaitop.depot.storage.service.SystemOutsideDataService;
import com.chinaitop.depot.storage.utils.ParameterUtil;
import com.chinaitop.depot.storage.utils.createKC;
import com.chinaitop.depot.utils.HttpUtil;
import com.fasterxml.jackson.databind.ObjectMapper;

@Service
public class StorageQualitycheckServiceImpl implements StorageQualitycheckService {

	@Resource
	private StorageQualitycheckMapper qualitycheckMapper;
	@Resource
	private StorageQualitycheckSonMapper storageQualitycheckSonMapper;
    @Resource
    private StorageQualitycheckSonService qualitycheckSonService;
    @Resource
    private StorageQualitycheckInspectionMapper qualityInspectionMapper;

    @Resource
    private AgileHouseFeignClient agileHouseFeignClient;

    @Resource
    private StorageHouseFeignClient storageHouseFeignClient;
    @Resource 
	private SystemOutsideDataService systemOutsideDataService;





    @Override
	public List<StorageQualitycheck> queryByQualitycheck(StorageQualitycheck example) {
		//对list进行处理获取仓房名称和货位名称
		return qualitycheckMapper.queryByExample(example);
	}

	// 保存数据
	@Override
	public void save(StorageQualitycheck qualitycheck, String qualitycheckInspectionJson) {
        try {
            // JSON字符串转对象
            ObjectMapper mapper = new ObjectMapper();
            qualitycheck.setSealStatus("0");    //默认0:未封仓   1:已封仓
            qualitycheck.setHistoryStatus(0);   //0:最新数据   1:历史数据
            Integer qualityCheckId=0;
            if (qualitycheck.getId() == null) {
                qualitycheckMapper.insertSelective(qualitycheck);// 新增数据
                if(qualitycheckInspectionJson.length() >2){
                    StorageQualitycheckInspection qualityInspection = (StorageQualitycheckInspection)mapper.readValue(qualitycheckInspectionJson, StorageQualitycheckInspection.class);
                    qualityCheckId = qualitycheck.getId()+1;
                    //新增质检
                    qualityInspection.setQualitycheckId(qualityCheckId);
                    qualityInspectionMapper.insertSelective(qualityInspection);
                }
            } else {
                qualitycheckMapper.updateByPrimaryKeySelective(qualitycheck);// 修改数据
                //修改质检
                if(null != qualitycheckInspectionJson && !qualitycheckInspectionJson.equals("null") && !qualitycheckInspectionJson.equals("")){
                    StorageQualitycheckInspection qualityInspection = (StorageQualitycheckInspection)mapper.readValue(qualitycheckInspectionJson, StorageQualitycheckInspection.class);
                    qualityCheckId = qualitycheck.getId();
                    qualityInspection.setQualitycheckId(qualityCheckId);
                    if(ParameterUtil.isnull(qualityInspection.getId())){
                        qualityInspectionMapper.insertSelective(qualityInspection);
                    }else {
                        qualityInspectionMapper.updateByPrimaryKeySelective(qualityInspection);
                    }

                }

            }
        } catch (IOException e) {
            e.printStackTrace();
        }
	}

	// 保存字表数据
	@Override
	public void saveOne(StorageQualitycheckSon qualitycheckone) {
	    if(qualitycheckone.getId() == null){
            storageQualitycheckSonMapper.insertSelective(qualitycheckone);// 新增数据
        } else {
            storageQualitycheckSonMapper.updateByPrimaryKeySelective(qualitycheckone);//修改数据
        }
	}

	@Override
	public void remove(int id) {
		// 删除数据
		qualitycheckMapper.deleteByPrimaryKey(id);
	}

	/**
	 * 根据条件获得第三方检查列表
	 * @param qualitycheck
	 * @return
	 */
	@Override
	public List<StorageQualitycheck> getThirdCheckList(StorageQualitycheck qualitycheck) {
		return  qualitycheckMapper.getThirdCheckList(qualitycheck);
	}

	@Override
	public List<StorageQualitycheck> getQualitycheckListByFoodbasicinfoCheck(StorageQualitycheck qualitycheck,Integer foodbasicinfoId) {

        List<StorageQualitycheck> list = qualitycheckMapper.getQualitycheckListByFoodbasicinfoCheck(qualitycheck);

        if(list.size()>0){
            int i = 0;
            for(;i<list.size();i++){
                List<StorageQualitycheckSon> QualitycheckdList =  qualitycheckSonService.selectByQualitycheckId(list.get(i).getId());
                for(StorageQualitycheckSon qc : QualitycheckdList){
                    if(qc.getNumbering()== 2789 ){
                        list.get(i).setEatScore(qc.getSetValue());
                        continue;
                    }
                    if(qc.getNumbering() == 2790 ){
                        list.get(i).setGlutenWaterCount(qc.getSetValue());
                        continue;
                    }
                }
            }
        }
		return list;
	}

	@Override
	public Map<String, Object> loadDataById(Integer id) {
        Map<String,Object> retMap = new HashMap<String,Object>();

        if(id!=null){
            StorageQualitycheck qualitycheck=qualitycheckMapper.queryPrimaryKey(id);
            retMap.put("qualitycheck",qualitycheck);
            if(qualitycheck!=null){
                StorageQualitycheckInspection qualitycheckInspection =qualityInspectionMapper.queryPrimaryKey(id);
                retMap.put("qualitycheckInspection",qualitycheckInspection);
            }
        }
        return retMap;
	}

    @Override
    public String getQualityStandardStr(Integer id) {
        String returnStr = "";
        StringBuffer sb = new StringBuffer();
        StorageQualitycheck qc = qualitycheckMapper.selectByPrimaryKey(id);
        if (qc != null) {
            // 品种.
            Integer subType = qc.getSubType();
            if (subType == 3164) {
                // 小麦.
                if (StringUtils.isNotBlank(qc.getCapacity())) {
                    sb.append("容重").append(qc.getCapacity()).append("g/L;");
                }
                if (StringUtils.isNotBlank(qc.getBadpart())) {
                    sb.append("不完善粒含量").append(qc.getBadpart()).append("%;");
                }

            } else if (subType == 3165) {
                // 玉米.
                if (StringUtils.isNotBlank(qc.getCapacity())) {
                    sb.append("容重").append(qc.getCapacity()).append("g/L;");
                }
                if (StringUtils.isNotBlank(qc.getBadpart())) {
                    sb.append("不完善粒含量总计").append(qc.getBadpart()).append("%;");
                }
                if (StringUtils.isNotBlank(qc.getMildewCount())) {
                    sb.append("其中：生霉粒").append(qc.getMildewCount()).append("%;");
                }
            } else if (subType == 3166) {
                // 稻谷.
                if (StringUtils.isNotBlank(qc.getHuskedRation())) {
                    sb.append("出糙率").append(qc.getMildewCount()).append("%;");
                }
                if (StringUtils.isNotBlank(qc.getFullGoodRation())) {
                    sb.append("整精米率").append(qc.getFullGoodRation()).append("%;");
                }
                if (StringUtils.isNotBlank(qc.getYellowPart())) {
                    sb.append("黄粒米").append(qc.getYellowPart()).append("%;");
                }
                if (StringUtils.isNotBlank(qc.getOutHuskedRation())) {
                    sb.append("谷外糙米").append(qc.getOutHuskedRation()).append("%;");
                }
            } else if (subType == 2777) {
                // 大豆.
                if (StringUtils.isNotBlank(qc.getGoodpart())) {
                    sb.append("完整粒率").append(qc.getGoodpart()).append("%;");
                }
                if (StringUtils.isNotBlank(qc.getDamageRatio())) {
                    sb.append("损伤粒率").append(qc.getDamageRatio()).append("%;");
                }
                if (StringUtils.isNotBlank(qc.getHotDamageRatio())) {
                    sb.append("其中：热损伤粒").append(qc.getHotDamageRatio()).append("%;");
                }
            }

            if (StringUtils.isNotBlank(qc.getImpurity())) {
                sb.append("杂质").append(qc.getImpurity()).append("%;");
            }
            if (StringUtils.isNotBlank(qc.getWater())) {
                sb.append("水分").append(qc.getWater()).append("%;");
            }
            if (StringUtils.isNotBlank(qc.getColorsmell())) {
                sb.append("色泽气味").append(qc.getColorsmell()).append(";");
            }
            returnStr = sb.toString();
        }
        return returnStr;
    }

    @Override
    public List<StorageQualitycheck> getHouseOrWareList(
            StorageQualitycheck qualitycheck) {
        List<StorageQualitycheck> list = null;
        //5：第三方检查
        qualitycheck.setTypeUpdate("5");

        // 仓房id.
        if (qualitycheck.getHouseId() != null) {
            list = qualitycheckMapper.getWareHouseIdList(qualitycheck);
        } else {
            list = qualitycheckMapper.getHouseIdList(qualitycheck);
        }
        return list;
    }

    /**
     * 更新质量管理为历史数据
     * @param houseId
     * @return
     */
    @Override
    public void setQualityCheckByHistoryStatus(Integer houseId,Integer warehouseId) {

        StorageQualitycheck qualitycheck = new StorageQualitycheck();
        //0:最新数据  1:历史数据
        qualitycheck.setHistoryStatus(1);

        StorageQualitycheckExample qualityCheckExample = new StorageQualitycheckExample();
        StorageQualitycheckExample.Criteria qualityCriteria = qualityCheckExample.createCriteria();
        if(houseId != null){
            qualityCriteria.andHouseIdEqualTo(houseId);
        }
        if(warehouseId != null){
            qualityCriteria.andWarehouseIdEqualTo(warehouseId);
        }

        //更新质量管理状态为历史数据
        qualitycheckMapper.updateByExampleSelective(qualitycheck,qualityCheckExample);

    }

    @Override
    public List<Map<String,Object>> getStoreQualityList(Integer orgId) {
        return qualitycheckMapper.getStoreQualityList(orgId);
    }
    

	@Override
	public List<Map<String,Object>> getTrdStoreQualityList() {
		// TODO Auto-generated method stub
		return qualitycheckMapper.getTrdStoreQualityList();
	}


    /**
     * 新增验收数据
     * @throws Exception
     */
    @Override
    @Transactional(readOnly = false)
    public List<StorageQualitycheck> getDataByZJId(Map<String, Object> paramMap) {
        return qualitycheckMapper.getDataByZJId(paramMap);
    }

    @Override
    public List<StorageQualitycheck> queryByQualitycheckData(StorageQualitycheck example) {

        return qualitycheckMapper.queryByExampleData(example);
    }

    @Override
    public List<StorageQualitycheck> queryByQualitycheckusiness(StorageQualitycheck qualityCheckExample) {

        return qualitycheckMapper.queryByQualitycheckusiness(qualityCheckExample);
    }

    @Override
    public List<Map<String,Object>>getAllQualityMain(Map<String, Object> paramMap) {
        return qualitycheckMapper.getAllQualityMain(paramMap);
    }

    @Override
    public List<Map<String, Object>> getAllQualityInfo(Map<String, Object> paramMap) {
        return qualitycheckMapper.getAllQualityInfo(paramMap);
    }

    @Override
    public Map<String, Object> getcheckByIds(Integer Id) {
        Map<String, Object> checkMap = qualitycheckMapper.getcheckById(Id);
        return checkMap;
    }

    @Override
    public List<Map<String, Object>> getAllFoodStocks(Map<String, Object> paramMap) {
        return qualitycheckMapper.getAllFoodStocks(paramMap);
    }

    @Override
    public JSONArray getHouseIdAll(Integer orgId) {

        List<JSONArray> ListAll = new ArrayList();

        JSONObject obj = new JSONObject();
        obj.put("unitid", orgId);
        String kcswStr = JSON.toJSONString(obj);

        //调用depot-agile接口
        JSONObject crkobj =  agileHouseFeignClient.getList(null, null, kcswStr, "ZJ");

        //调用depot-storage接口
        JSONObject cpyobj =  storageHouseFeignClient.getNumberList(null, null, null, null, orgId);

        JSONArray crkJson = crkobj.getJSONArray("list");
        JSONArray cpyJson = cpyobj.getJSONArray("list");

        if(crkJson.size() > 0 && cpyJson.size() <= 0 ){
            ListAll.addAll(Collections.singleton(crkJson));
        }

        if(crkJson.size() <= 0 && cpyJson.size() > 0){
            for(int j=0;j<cpyJson.size();j++){
                cpyJson.getJSONObject(j).put("ch",cpyJson.getJSONObject(j).get("houseId"));
            }
            ListAll.addAll(Collections.singleton(cpyJson));
        }

        if(crkJson.size() > 0 && cpyJson.size() > 0 ){
            for(int i=0;i<crkJson.size();i++) {
                for(int j=0;j<cpyJson.size();j++){
                    cpyJson.getJSONObject(j).put("ch",cpyJson.getJSONObject(j).get("houseId"));
                    if(crkJson.getJSONObject(i).get("ch").toString().equals(cpyJson.getJSONObject(j).get("houseId"))){
                        cpyJson.remove(j);
                        j--;
                    }
                }
            }
            ListAll.addAll(Collections.singleton(crkJson));
            ListAll.addAll(Collections.singleton(cpyJson));
        }

        return JSONObject.parseArray(String.valueOf(ListAll));
    }

    @Override
	public List<StorageQualitycheck> findByCondition(StorageQualitycheckExample example) {
		return qualitycheckMapper.selectByExample(example);
	}

	@Override
	public int deleteStorageQualitycheckInspection(StorageQualitycheckInspectionExample example) {
		return qualityInspectionMapper.deleteByExample(example);
	}

	@Override
	public void pushQualitycheck_spt(StorageQualitycheck qualitycheck, String reportPath) throws Exception {
		//自检或出库 保存或修改数据 把数据推给市平台
    	if (null != qualitycheck.getId()) {
    		Map<String, Object> dataMap = getcheckByIds(qualitycheck.getId());
    		String url = reportPath+"/zjbg/receiveData";//市级接口
    		String strResult = HttpUtil.doPost(url, JSON.toJSONString(dataMap));
    		systemOutsideDataService.addlogger("zjts_spt", JSON.toJSONString(dataMap), strResult);
    	}
	}

	@Override
	public void pushQualitycheckLsdj_crk(StorageQualitycheck qualitycheck, String ws_path) throws Exception {
		if ("0".equals(qualitycheck.getTypeUpdate()) && qualitycheck.getProcessState() == 3) {//只有粮食自检,并且是已提交的数据才走这里
			String crk_yjd_url = qualitycheckMapper.findConditionCrkYjdUrl(qualitycheck.getOrgId().toString());
			if (StringUtils.isNotBlank(crk_yjd_url)) {
				//拼接完整URL
				int end = crk_yjd_url.indexOf("/Base");
				String url = crk_yjd_url.substring(0, end);
				StringBuffer sbf = new StringBuffer(50);
				sbf.append(url).append(ws_path);

				//拼接参数
				Object[] parameters = new Object[4];
				parameters[0] = qualitycheck.getOrgId().toString();//组织机构ID
				parameters[1] = qualitycheck.getHouseId().toString();//仓房
				parameters[2] = qualitycheck.getWarehouseId().toString();//货位
				parameters[3] = qualitycheck.getYpdj().toString();//样品等级
				
				String strResult = createKC.createKC("ChangeGrainGrade", sbf.toString(), parameters);
				systemOutsideDataService.addlogger("ts_crk_lsdj", JSON.toJSONString(parameters), strResult);
			}
		}
	}

	@Override
	public List<StorageQualitycheck> getList(Integer houseId, Integer wareId, String checkType, String checkResult,
			Integer historyStatus, Integer orgId, Integer processState, String isdx, String checkTime) throws Exception {
		StorageQualitycheckExample example = new StorageQualitycheckExample();
    	Criteria criteria = example.createCriteria();

    	//0:初检 1：验收 2：质量普查 3：出库, 4:春秋普查,5:第三方检验
        if (StringUtils.isNotBlank(checkType)){
        	criteria.andTypeUpdateEqualTo(checkType);
        }
        // 仓房id.
        if (houseId != null) {
        	criteria.andHouseIdEqualTo(houseId);
        }
        // 货位id.
        if (wareId != null) {
        	criteria.andWarehouseIdEqualTo(wareId);
        }
        // 粮库id.
        if (orgId != null) {
        	criteria.andOrgIdEqualTo(orgId);
        }

        //1扦样化验待处理 2化验已处理,3:化验已保存
        if (null != processState) {
        	criteria.andProcessStateEqualTo(processState);
        }

        //检验结果（0：合格 , 1 :不合格）
        if (StringUtils.isNotBlank(checkResult)) {
        	criteria.andCheckResultEqualTo(checkResult);
        }
        //数据状态(0:最新数据  1:历史数据)
        if (null != historyStatus) {
        	criteria.andHistoryStatusEqualTo(historyStatus);
        }
        
        //按照isdx查询检验时间范围
        if (StringUtils.isNotBlank(isdx)) {
        	if ("0".equals(isdx) && StringUtils.isNotBlank(checkTime)) {
        		criteria.andCheckTimeGreaterThan(ParameterUtil.string2datetime(checkTime));
        	}
        	if ("1".equals(isdx) && StringUtils.isNotBlank(checkTime)) {
        		criteria.andCheckTimeLessThan(ParameterUtil.string2datetime(checkTime));
        	}
        }

        //查询从质量管理功能里面添加到数据库的数据
        criteria.andFlagTypeIsNull();

        example.setOrderByClause(" check_time desc");

        List<StorageQualitycheck> list = qualitycheckMapper.selectByExample(example);

		return list;
	}

	@Override
	public Map<String, Object> queryCheckDataDetail(Integer houseId, Integer wareId, String checkType,
			Integer historyStatus, Integer orgId, Integer processState) throws Exception {
		List<StorageQualitycheck> list = getList(houseId, wareId, checkType, null, historyStatus, orgId, processState, null , null);
		Map<String, Object> retMap = null;
		if (null != list && list.size() > 0) {
			Integer id = list.get(0).getId();
			retMap = loadDataById(id);
		}
		return retMap;
	}

	@Override
	public Map<String, Object> findConditionCkCheck(Integer houseId, Integer wareId, String checkType) throws Exception {

		//查询最近的封仓时间
		Map<String, Object> paramMap = new HashMap<>();
		if (null != houseId) {
			paramMap.put("houseId", houseId);
		}
		if (null != wareId) {
			paramMap.put("warehouseId", wareId);
		}
		String fcsj = qualitycheckMapper.findConditionFcsj(paramMap);

		Map<String, Object> resultMap = new HashMap<>();
		if (StringUtils.isNotBlank(fcsj)) {
			List<StorageQualitycheck> list = getList(houseId, wareId, checkType, null, null, null, 3, "0" , fcsj);
			if (null != list && list.size() > 0) {
				resultMap.put("status", "200");
				resultMap.put("msg", list);
			} else {
				resultMap.put("status", "500");
				resultMap.put("msg", "未查到封仓之后的出库质检数据");
			}
		} else {
			resultMap.put("status", "500");
			resultMap.put("msg", "连封仓数据都没查到");
		}

		return resultMap;
	}
}